package parameters_test

import (
	. "github.com/onsi/ginkgo"
	. "github.com/onsi/gomega"

	. "k8s.libre.sh/parameters"

	corev1 "k8s.io/api/core/v1"
)

var _ = Describe("Parameters", func() {
	var (
		paramEnvVar           Parameter
		paramEnvVarFrom       Parameter
		paramEnvFrom          Parameter
		paramEnvFromSecret    Parameter
		paramEnvVarFromSecret Parameter
		valueFrom             ValueFrom
		valueFromSecret       ValueFrom
		parameters            *Parameters
	)

	BeforeEach(func() {
		paramEnvVar = Parameter{
			Key:       "MY_PARAMETER",
			Value:     "Parameter",
			Type:      "ConfigMap",
			MountType: MountLiteral,
		}

		valueFrom = ValueFrom{
			FromKey: "FROM_PARAMETER",
			Source:  "source",
		}

		valueFromSecret = ValueFrom{
			FromKey: "FROM_SECRET_PARAMETER",
			Source:  "secret_source",
		}

		paramEnvVarFrom = Parameter{
			Key:       "MY_PARAMETER",
			ValueFrom: valueFrom,
			Type:      "ConfigMap",
			MountType: MountFrom,
		}

		paramEnvVarFromSecret = Parameter{
			Key:       "MY_SECRET_PARAMETER",
			ValueFrom: valueFromSecret,
			Type:      "Secret",
			MountType: MountFrom,
		}

		paramEnvFrom = Parameter{
			ValueFrom: ValueFrom{
				Source: "source",
			},
			Type:      "ConfigMap",
			MountType: MountFrom,
		}

		paramEnvFromSecret = Parameter{
			ValueFrom: ValueFrom{
				Source: "secret_source",
			},
			Type:      "Secret",
			MountType: MountFrom,
		}

		parameters = &Parameters{
			&paramEnvVar,
			&paramEnvVarFrom,
			&paramEnvVarFromSecret,
			&paramEnvFrom,
			&paramEnvFromSecret,
		}

	})

	Describe("Generating Envirnonment Variables for containers from parameter", func() {
		Context("MountType is Literal", func() {
			It("should generate envVar", func() {

				expectedObj := corev1.EnvVar{
					Name:  "MY_PARAMETER",
					Value: "Parameter",
				}

				res, err := paramEnvVar.GetEnvVar()

				Expect(res).To(Equal(expectedObj))
				Expect(err).NotTo(HaveOccurred())
			})
		})
		Context("MountType is Template", func() {
			It("should not generate envVar", func() {

				//				expectedObj := corev1.EnvVar{
				//					Name:  "MY_PARAMETER",
				//					Value: "Parameter",
				//				}
				paramEnvVar.MountType = MountTemplate

				_, err := paramEnvVar.GetEnvVar()

				//	Expect(res).To(Equal(expectedObj))
				Expect(err).To(HaveOccurred())
			})
		})

		Context("MountType is From", func() {
			It("should generate envVar with valueFrom", func() {
				expectedObj := corev1.EnvVar{
					Name: "MY_PARAMETER",
					ValueFrom: &corev1.EnvVarSource{
						ConfigMapKeyRef: &corev1.ConfigMapKeySelector{
							LocalObjectReference: corev1.LocalObjectReference{
								Name: "source",
							},
							Key: "FROM_PARAMETER",
						},
					},
				}

				res, err := paramEnvVarFrom.GetEnvVar()

				Expect(res).To(Equal(expectedObj))
				Expect(err).NotTo(HaveOccurred())
			})
		})

		Context("With from key", func() {
			It("should generate envFrom", func() {

				expectedObj := corev1.EnvFromSource{
					ConfigMapRef: &corev1.ConfigMapEnvSource{
						LocalObjectReference: corev1.LocalObjectReference{
							Name: "source",
						},
					},
				}

				res, err := paramEnvFrom.GetEnvFrom()

				Expect(res).To(Equal(expectedObj))
				Expect(err).NotTo(HaveOccurred())
			})
		})
	})

	Describe("Generating Env for containers with Parameters", func() {
		Context("Generating env", func() {
			It("should generate env", func() {

				expectedObj := []corev1.EnvVar{
					{
						Name:  "MY_PARAMETER",
						Value: "Parameter",
					},
					{
						Name: "MY_PARAMETER",
						ValueFrom: &corev1.EnvVarSource{
							ConfigMapKeyRef: &corev1.ConfigMapKeySelector{
								LocalObjectReference: corev1.LocalObjectReference{
									Name: "source",
								},
								Key: "FROM_PARAMETER",
							},
						},
					},
					{
						Name: "MY_SECRET_PARAMETER",
						ValueFrom: &corev1.EnvVarSource{
							SecretKeyRef: &corev1.SecretKeySelector{
								LocalObjectReference: corev1.LocalObjectReference{
									Name: "secret_source",
								},
								Key: "FROM_SECRET_PARAMETER",
							},
						},
					},
				}

				res := parameters.GetEnv()

				Expect(res).To(Equal(expectedObj))
				//	Expect(err).NotTo(HaveOccurred())
			})
			It("should generate envFrom", func() {

				expectedObj := []corev1.EnvFromSource{
					{
						ConfigMapRef: &corev1.ConfigMapEnvSource{
							LocalObjectReference: corev1.LocalObjectReference{
								Name: "source",
							},
						},
					},
					{
						SecretRef: &corev1.SecretEnvSource{
							LocalObjectReference: corev1.LocalObjectReference{
								Name: "secret_source",
							},
						},
					},
				}

				res := parameters.GetEnvFrom()

				Expect(res).To(Equal(expectedObj))
				//	Expect(err).NotTo(HaveOccurred())
			})
		})
		Context("Generating data", func() {
			It("should generate data", func() {

				expectedObj := map[string]string{
					"MY_PARAMETER": "Parameter",
				}

				Expect(parameters.GetData()).To(Equal(expectedObj))
				//	Expect(err).NotTo(HaveOccurred())
			})
		})
	})
})
